from .TransactionService import TransactionService


class GetTransactionService(TransactionService):
    def __init__(self, store):
        super(GetTransactionService, self).__init__(store)

        self.tid = None
        self.reference = None
        self.refunds = False

    def get_uri(self):
        if self.reference is not None:
            return "{}?reference={}".format(super().get_uri(), self.reference)

        if self.tid is None:
            raise ValueError("You need to specify one: the tid or the reference")

        if self.refunds:
            return "{}/{}/refunds".format(super().get_uri(), self.tid)

        return "{}/{}".format(super().get_uri(), self.tid)

    def execute(self):
        self.send_request(TransactionService.GET)



from .Additional import Additional
from .Address import Address
from .Antifraud import Antifraud
from .Authorization import Authorization
from .Capture import Capture
from .Cart import Cart
from .Consumer import Consumer
from .Document import Document
from .Environment import Environment
from .eRede import eRede
from .Flight import Flight
from .Iata import Iata
from .Item import Item
from .Passenger import Passenger
from .Phone import Phone
from .RedeError import RedeError
from .Refund import Refund
from .Store import Store
from .SubMerchant import SubMerchant
from .ThreeDSecure import ThreeDSecure
from .Transaction import Transaction
from .Url import Url



from .RedeSerializable import RedeSerializable


class Url(RedeSerializable):
    CALLBACK = "callback"
    THREE_D_SECURE_FAILURE = "threeDSecureFailure"
    THREE_D_SECURE_SUCCESS = "threeDSecureSuccess"

    def __init__(self, url, kind=None):
        if kind is None:
            kind = Url.CALLBACK

        self.url = url
        self.kind = kind

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        return Url(dict.get("url", ""), dict.get("kind", ""))



from .RedeSerializable import RedeSerializable


class Antifraud(RedeSerializable):
    def __init__(self):
        self.recommendation = None
        self.riskLevel = None
        self.score = None
        self.success = False

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Antifraud()

        for k, v in dict.items():
            setattr(instance, k, v)

        return instance



class Capture:
    def __init__(self):
        self.amount = None
        self.dateTime = None
        self.nsu = None

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Capture()

        for k, v in dict.items():
            setattr(instance, k, v)

        return instance



from .service import *


class eRede:
    USER_AGENT = "eRede/1.0 (SDK; Python)"

    def __init__(self, store):
        """

        :type store: `erede.Store.Store`
        """
        self.store = store

    def authorize(self, transaction):
        return self.create(transaction)

    def create(self, transaction):
        """Create a transaction

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """
        create_transaction = CreateTransactionService(self.store, transaction)

        return create_transaction.execute()

    def cancel(self, transaction):
        """Cancel a Transaction

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        cancel_transaction = CancelTransactionService(self.store, transaction)

        return cancel_transaction.execute()

    def capture(self, transaction):
        """Capture a Transaction

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        capture_transaction = CaptureTransactionService(self.store, transaction)

        return capture_transaction.execute()

    def get_by_tid(self, tid):
        """Get a Transaction by its TID

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        get_transaction = GetTransactionService(self.store)
        get_transaction.tid = tid

        return get_transaction.execute()

    def get_by_reference(self, reference):
        """Get a Transaction by its reference

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        get_transaction = GetTransactionService(self.store)
        get_transaction.reference = reference

        return get_transaction.execute()

    def get_refunds(self, tid):
        """Get a Transaction refunds

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        get_transaction = GetTransactionService(self.store)
        get_transaction.tid = tid
        get_transaction.refunds = True

        return get_transaction.execute()

    def zero(self, transaction):
        """Verify a card

        :type transaction: `erede.Transaction.Transaction`
        :rtype: `erede.Transaction.Transaction`
        """

        amount = transaction.amount
        capture = transaction.capture

        transaction.amount = 0
        transaction.capture = True

        transaction = self.create(transaction)

        transaction.amount = amount
        transaction.capture = capture

        return transaction



from .RedeSerializable import RedeSerializable


class Additional(RedeSerializable):
    def __init__(self, gateway=None, module=None):
        self.gateway = gateway
        self.module = module

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Additional()

        for k, v in dict.items():
            setattr(instance, k, v)

        return instance



from .TransactionService import TransactionService


class CreateTransactionService(TransactionService):
    def __init__(self, store, transaction):
        """

        :type store: `erede.Store.Store`
        :type transaction: `erede.Transaction.Transaction`
        """
        super(CreateTransactionService, self).__init__(store)

        self.transaction = transaction

    def execute(self):
        return self.send_request(TransactionService.POST, self.transaction.to_json())




class Authorization:
    def __init__(self):
        self.affiliation = None
        self.amount = None
        self.authorizationCode = None
        self.cardBin = None
        self.cardHolderName = None
        self.dateTime = None
        self.installments = None
        self.kind = None
        self.last4 = None
        self.nsu = None
        self.origin = None
        self.reference = None
        self.returnCode = None
        self.returnMessage = None
        self.status = None
        self.subscription = None
        self.tid = None

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Authorization()

        for k, v in dict.items():
            setattr(instance, k, v)

        return instance



import json


class RedeSerializable:
    def to_json(self):
        return json.dumps(self.serialize(), cls=RecursiveEncoder)

    def serialize(self):
        dict = {k: v for k, v in self.__dict__.items() if v is not None}

        for k, v in dict.items():
            if hasattr(v, "serialize"):
                dict[k] = v.serialize()

        return dict


class RecursiveEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'serialize'):
            return obj.serialize()
        else:
            return json.JSONEncoder.default(self, obj)



from .Address import Address
from .Consumer import Consumer
from .Iata import Iata
from .RedeSerializable import RedeSerializable


class Cart(RedeSerializable):
    def __init__(self):
        self.billing = None
        self.consumer = None
        self.environment = None
        self.iata = None
        self.items = None
        self.shipping = None

    def adress(self, address_type=None):
        """

        :type address_type: `int`
        """
        if address_type is None:
            address_type = Address.BOTH

        address = Address()

        if (address_type & Address.BILLING) == Address.BILLING:
            self.billing = address

        if (address_type & Address.SHIPPING) == Address.SHIPPING:
            self.shipping = [address]

        return address

    def add_item(self, item):
        """

        :type item: `erede.Item.Item`
        """
        if self.items is None:
            self.items = []

        self.items.append(item)

        return self

    def consumer(self, name, email, cpf):
        """

        :type name: `string`
        :type email: `string`
        :type cpf: `string`
        :rtype: `erede.Consumer.Consumer`
        """
        self.consumer = Consumer(name, email, cpf)

        return self.consumer

    def set_iata(self, flight):
        """

        :type flight: `erede.Flight.Flight`
        """
        self.iata = Iata()
        self.iata.add_flight(flight)

        return self



from .Phone import Phone
from .RedeSerializable import RedeSerializable


class Passenger(RedeSerializable):
    def __init__(self, name, email, ticket):
        self.name = name
        self.email = email
        self.ticket = ticket
        self.phone = None

    def set_phone(self, ddd, number, phone_type=None):
        if phone_type is None:
            phone_type = Phone.CELLPHONE

        self.phone = Phone(ddd, number, phone_type)



from .RedeSerializable import RedeSerializable


class Environment(RedeSerializable):
    PRODUCTION = "https://api.userede.com.br/erede"
    SANDBOX = "https://api.userede.com.br/desenvolvedores"
    VERSION = "v1"

    def __init__(self, base_url, version=None):
        if version is None:
            version = Environment.VERSION

        self.ip = None
        self.sessionId = None
        self.endpoint = "{}/{}".format(base_url, version)

    @staticmethod
    def production():
        """

        :rtype: `erede.Environment.Environment`
        """
        return Environment(Environment.PRODUCTION)

    @staticmethod
    def sandbox():
        """

        :rtype: `erede.Environment.Environment`
        """
        return Environment(Environment.SANDBOX)



from .RedeSerializable import RedeSerializable


class Flight(RedeSerializable):
    def __init__(self, number, flight_from, to, date):
        self.number = number
        self.From = flight_from
        self.to = to
        self.date = date
        self.passenger = None

    def add_passenger(self, passenger):
        if self.passenger is None:
            self.passenger = []

        self.passenger.append(passenger)

        return self



from .Document import Document
from .Phone import Phone
from .RedeSerializable import RedeSerializable


class Consumer(RedeSerializable):
    MALE = "M"
    FEMALE = "F"

    def __init__(self, name, email, cpf):
        """

        :type name: `string`
        :type email: `string`
        :type cpf: `string`
        """
        self.cpf = cpf
        self.email = email
        self.name = name
        self.documents = None
        self.gender = None
        self.phone = None

    def add_document(self, document_type, number):
        """

        :type document_type: `string`
        :type number: `string`
        :rtype: `erede.Document.Document`
        """
        if self.documents is None:
            self.documents = []

        document = Document(document_type, number)

        self.documents.append(document)

        return document

    def add_phone(self, ddd, number, phone_type=None):
        if phone_type is None:
            phone_type = Phone.CELLPHONE

        phone = Phone(ddd, number, phone_type)

        self.phone = phone

        return phone



from .RedeSerializable import RedeSerializable


class Iata(RedeSerializable):
    def __init__(self, code, departure_tax):
        self.code = code
        self.departureTax = departure_tax
        self.flight = None

    def add_flight(self, flight):
        if self.flight is None:
            self.flight = []

        self.flight.append(flight)

        return self



from .TransactionService import TransactionService


class CaptureTransactionService(TransactionService):
    def __init__(self, store, transaction):
        """

        :type store: `erede.Store.Store`
        :type transaction: `erede.Transaction.Transaction`
        """
        super(CaptureTransactionService, self).__init__(store)

        self.transaction = transaction

    def get_uri(self):
        return "{}/{}".format(super().get_uri(), self.transaction.tid)

    def execute(self):
        self.send_request(TransactionService.PUT, self.transaction.to_json())



from .RedeSerializable import RedeSerializable


class Phone(RedeSerializable):
    CELLPHONE = 1
    HOME = 2
    WORK = 3
    OTHER = 4

    def __init__(self, ddd, number, phone_type=None):
        if phone_type is None:
            phone_type = Phone.CELLPHONE

        self.ddd = ddd
        self.number = number
        self.type = phone_type



from .RedeSerializable import RedeSerializable


class Address(RedeSerializable):
    BILLING = 1
    SHIPPING = 2
    BOTH = 3
    APARTMENT = 1
    HOUSE = 2
    COMMERCIAL = 3
    OTHER = 4

    def __init__(self):
        self.address = None
        self.addresseeName = None
        self.city = None
        self.number = None
        self.state = None
        self.type = None
        self.zipCode = None




from .Environment import Environment


class Store:
    def __init__(self, filliation, token, environment=None):
        """

        :type filliation: `string`
        :type token: `string`
        :type environment: `erede.Environment.Environment`
        """
        if environment is None:
            environment = Environment.production()

        self.filliation = filliation
        self.token = token
        self.environment = environment



from .Additional import Additional
from .Antifraud import Antifraud
from .Authorization import Authorization
from .Cart import Cart
from .Environment import Environment
from .RedeSerializable import RedeSerializable
from .ThreeDSecure import ThreeDSecure
from .Url import Url
from .Refund import Refund
from .Capture import Capture
from .Iata import Iata


class Transaction(RedeSerializable):
    CREDIT = "credit"
    DEBIT = "debit"

    ORIGIN_EREDE = 1
    ORIGIN_VISA_CHECKOUT = 4
    ORIGIN_MASTERPASS = 6

    def __init__(self, amount, reference):
        """

        :type amount: `int`
        :type reference: `string`
        """

        """
        
        """
        self.amount = round(amount*100)
        self.reference = reference
        self.additional = None
        self.antifraud = None
        self.antifraudRequired = None
        self.authorization = None
        self.authorizationCode = None
        self.cancelId = None
        self.capture = None
        self.cardBin = None
        self.cardHolderName = None
        self.cardNumber = None
        self.cart = None
        self.dateTime = None
        self.distributorAffiliation = None
        self.expirationMonth = None
        self.expirationYear = None
        self.iata = None
        self.installments = None
        self.kind = None
        self.last4 = None
        self.nsu = None
        self.origin = None
        self.refundDateTime = None
        self.refundId = None
        self.refunds = None
        self.requestDateTime = None
        self.returnCode = None
        self.returnMessage = None
        self.securityCode = None
        self.softDescriptor = None
        self.storageCard = None
        self.subscription = None
        self.threeDSecure = None
        self.tid = None
        self.urls = None
        self.subMerchant = None
        self.paymentFacilitatorID = None

    def add_url(self, url, kind=None):
        if kind is None:
            kind = Url.CALLBACK

        if self.urls is None:
            self.urls = []

        self.urls.append(Url(url, kind))

        return self

    def set_antifraud(self, environment=None):
        if environment is None:
            environment = Environment.production()

        cart = Cart()
        cart.environment = environment

        self.antifraudRequired = True
        self.cart = cart

        return cart

    def credit_card(self, card_number, security_code, expiration_month, expitarion_year, card_holder_name):
        return self.card(card_number, security_code, expiration_month, expitarion_year, card_holder_name,
                         Transaction.CREDIT)

    def debit_card(self, card_number, security_code, expiration_month, expitarion_year, card_holder_name):
        return self.card(card_number, security_code, expiration_month, expitarion_year, card_holder_name,
                         Transaction.DEBIT)

    def card(self, card_number, security_code, expiration_month, expitarion_year, card_holder_name, kind):
        self.cardNumber = card_number
        self.securityCode = security_code
        self.expirationMonth = expiration_month
        self.expirationYear = expitarion_year
        self.cardHolderName = card_holder_name
        self.kind = kind

        return self

    def capture_transaction(self, capture=True):
        if not capture and self.kind == Transaction.DEBIT:
            raise ValueError("Debit transactions will always be captured")

        self.capture = capture

        return self

    def mcc(self, soft_descriptor, payment_facilitator_id, sub_merchant):
        self.softDescriptor = soft_descriptor
        self.paymentFacilitatorID = payment_facilitator_id
        self.subMerchant = sub_merchant

        return self

    def three_d_secure(self, on_failure=ThreeDSecure.DECLINE_ON_FAILURE, embed=True, transaction_id="", indicator="1"):
        three_d_secure = ThreeDSecure()
        three_d_secure.onFailure = on_failure
        three_d_secure.embedded = embed
        three_d_secure.directoryServerTransactionId = transaction_id
        three_d_secure.threeDIndicator = indicator

        self.threeDSecure = three_d_secure

        return self

    def set_additional(self, gateway=None, module=None):
        """

        :type gateway: `int`
        :type module: `int`
        :rtype: `erede.Transaction.Transaction`
        """
        self.additional = Additional(gateway, module)

        return self

    def set_iata(self, code, departure_tax):
        self.iata = Iata(code, departure_tax)

        return self

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Transaction(dict.get("amount", 0), dict.get("reference", ""))

        for k, v in dict.items():
            if k == "refunds":
                instance.refunds = []

                for r in dict.get("refunds", []):
                    instance.refunds.append(Refund.unserialize(r))

                continue

            if k == "urls":
                instance.urls = []

                for u in dict.get("urls", []):
                    instance.urls.append(Url.unserialize(u))

                continue

            if k == "capture":
                instance.capture = Capture.unserialize(v)

                continue

            if k == "authorization":
                instance.authorization = Authorization.unserialize(v)

                continue

            if k == "threeDSecure":
                instance.threeDSecure = ThreeDSecure.unserialize(v)

                continue

            if k == "antifraud":
                instance.antifraud = Antifraud.unserialize(v)

                continue

            if k == "additional":
                instance.additional = Additional.unserialize(v)

                continue

            setattr(instance, k, v)

        return instance



from .RedeSerializable import RedeSerializable


class Item(RedeSerializable):
    PHYSICAL = 1
    DIGITAL = 2
    SERVICE = 3
    AIRLINE = 4

    def __init__(self, item_id, quantity, item_type=None):
        if item_type is None:
            item_type = Item.PHYSICAL

        self.id = item_id
        self.quantity = quantity
        self.type = item_type
        self.amount = None
        self.description = None
        self.discount = None
        self.freight = None
        self.shippingType = None



import requests
import erede

from erede.Transaction import Transaction
from erede.RedeError import RedeError


class TransactionService:
    GET = "get"
    POST = "post"
    PUT = "put"

    def __init__(self, store):
        """

        :type store: `erede.Store.Store`
        """
        self.store = store

    def execute(self):
        raise NotImplementedError("Not implemented")

    def get_uri(self):
        return "{}/transactions".format(self.store.environment.endpoint)

    def send_request(self, method, body=None):
        headers = {'User-Agent': "{} Store/{}".format(erede.eRede.USER_AGENT, self.store.filliation),
                   "Accept": "application/json",
                   "Content-Type": "application/json"}

        response = getattr(requests, method)(self.get_uri(),
                                             auth=(self.store.filliation, self.store.token),
                                             data=body,
                                             headers=headers)
        '''
        :type response: `requests.Response`
        '''

        if response.status_code >= 400:
            error = response.json()

            raise RedeError(error.get("returnMessage", "opz"), error.get("returnCode", 0))

        return Transaction.unserialize(response.json())



from .RedeSerializable import RedeSerializable


class SubMerchant(RedeSerializable):
    def __init__(self, mcc, city, country):
        self.mcc = mcc
        self.city = city
        self.country = country



from .RedeSerializable import RedeSerializable


class ThreeDSecure(RedeSerializable):
    CONTINUE_ON_FAILURE = "continue"
    DECLINE_ON_FAILURE = "decline"

    def __init__(self):
        self.cavv = None
        self.eci = None
        self.embedded = True
        self.onFailure = None
        self.url = None
        self.userAgent = eRede.USER_AGENT
        self.xid = None
        self.threeDIndicator = "1"
        self.directoryServerTransactionId = None

    @staticmethod
    def unserialize(dict=None):
        if dict is None:
            dict = {}

        instance = Additional()

        for k, v in dict.items():
            setattr(instance, k, v)

        return instance
